#include "itext.h"

#include <sys/stat.h>
#include <wkf.h>

extern char *XJP_MESSAGE[];

static unsigned char *HEADER_LINE = "---------------------------";

unsigned int SPACE = ' ';
unsigned int CHU_KEY_C1 = 0xa2;
unsigned int CHU_KEY_C2 = 0xb0;

static IBUF_T *ibuf_data_alloc()
{
	IBUF_T *it;

	it = (IBUF_T *)malloc(sizeof(IBUF_T));

	it->line = 0;
	it->i_head = NULL;
	it->i_tail = NULL;

	return it;
}
static void ibuf_data_add(IBUF_T **start,IBUF_T **end,IBUF_T *mg)
{
	if(*end){
		mg->next = NULL;
		mg->prev = *end;

		(*end)->next = mg;
		*end = mg;
	}else{
		mg->next = NULL;
		mg->prev = NULL;

		*end = mg;
		*start = mg;
	}
}
void ibuf_data_free(IBUF_T **start,IBUF_T **end)
{
	IBUF_T *ib,*ibb;
	ITEXT_T *it,*itt;

	if(*start == NULL) return;

	for(ib = *start;ib!=NULL;ib=ibb){
		ibb = ib->next;

		itext_data_free(ib->i_head);

		free(ib);
	}

	*start = NULL;
	*end = NULL;

	return;
}
void itext_data_free(ITEXT_T *ihead)
{
	ITEXT_T *it,*itt;

	if(ihead == NULL) return;

	for(it=ihead;it!=NULL;it=itt){
		itt = it->next;
		if(it->chu_string != NULL) free(it->chu_string);
		if(it->rubi_head != NULL) itext_data_free(it->rubi_head);
		if(it->chu_head != NULL) itext_data_free(it->chu_head);

		free(it);
	}

	return ;
}
static ITEXT_T *itext_data_alloc()
{
	ITEXT_T *it;

	it = (ITEXT_T *)malloc(sizeof(ITEXT_T));
	it->c1 = SPACE;
	it->c2 = SPACE;
	it->chu_string = NULL;

	it->x_off = 0;
	it->y_off = 0;

	it->flag = 0;

	it->ej = 0;
	it->chu = 0;
	it->rub = -1;
	it->jis = -1;
	it->tes = -1;
	it->bouten = -1;
	it->bousen = -1;
	it->small = 0;

	it->rubi_head = NULL;
	it->rubi_tail = NULL;

	it->chu_head = NULL;
	it->chu_tail = NULL;

	return it;
}
static void itext_data_add(ITEXT_T **start,ITEXT_T **end,ITEXT_T *mg)
{
	if(*end){
		mg->next = NULL;
		mg->prev = *end;

		(*end)->next = (ITEXT_T *)mg;
		*end = mg;
	}else{
		mg->next = NULL;
		mg->prev = NULL;

		*end = mg;
		*start = mg;
	}
}
static int _change_c(unsigned int *c1,unsigned int *c2,unsigned int *c3)
{
	switch(*c2){
		case 0x2d: /* - */
			*c2 = 0x7c; /* | */
		break;
	}

	return 0;
}
static int _ibuf_add(	IBUF_T **head,
			IBUF_T **tail,
			ITEXT_T **ihead,
			ITEXT_T **itail,
			int i,
			int line	)
{
	IBUF_T *ib;

	ib = (IBUF_T *)ibuf_data_alloc();

	ib->line = line;
	ib->i_head = *ihead;
	ib->i_tail = *itail;
	ibuf_data_add(head,tail,ib);

	*ihead = NULL;
	*itail = NULL;

	return 0;
}
static int _no_k(unsigned char *s)
{
	int i;
	unsigned char *p;

	for(p=s,i=0;*p;p++,i++){
		if(*p == '\r' || *p == '\n'){
			*p = '\0';
			break;
		}
	}

	return i;
}
int _checkFileType(char *fname)
{
	char *p;
	int i;

	for(i=strlen(fname)-1;i>0;i--){
		if(*(fname+i) == '/') break;
	}
	for(p=fname+i;*p;p++){
		if(*p == '.') break;
	}

	if(strcmp(p,".txt")==0 || strcmp(p,".TXT")==0 || strcmp(p,".text")==0 || strcmp(p,".TEXT")==0) return 0;

	if(strcmp(p,".zip")==0 || strcmp(p,".ZIP")==0 || strcmp(p,".Zip")==0) return 1;

	return -1;

}
int itextDataInsert(		ITEXT_T **ihead,
				ITEXT_T **itail,
				unsigned int c1,
				unsigned int c2,
				int ej,
				int chu,
				int small		)
{
	ITEXT_T *it;
	char temp[3];

	if(ej == 1){
		temp[0] = c1;
		temp[1] = c2;
		temp[2] = '\0';
	}

	it = (ITEXT_T *)itext_data_alloc();
	it->c1 = c1;
	it->c2 = c2;
	it->ej = ej;
	it->chu = chu;
	it->small = small;
	itext_data_add(ihead,itail,it);

	return 0;
}
int itextGetTextData(char *fname,IINFO_T *info,IBUF_T **head,IBUF_T **tail,int ruby_on)
{
	struct stat sbuf;

	FILE *fp;
	unsigned char *p;
	unsigned char buf[BUFSIZ];
	unsigned char buf_euc[BUFSIZ];

	IBUF_T *ib;
	ITEXT_T *it,*rb,*chu;
	ITEXT_T *ihead = NULL;
	ITEXT_T *itail = NULL;

	unsigned int c1;
	unsigned int c2;

	int slen,i;
	int total_size = 0;
	int total_line = 1;
	int head_line = 0;
	int file_type;
	char zipfname[256];

	int HEAD = 0;
	int RUBI = 0;
	int CHU = 0;
	int SMALL = 0;
	int RUB = -1;


	file_type = _checkFileType(fname);

	if(file_type == 0){
		if((fp = fopen(fname,"r")) == NULL) return -1;

	}else if(file_type == 1){
		if(itextUnZipFile(fname,zipfname,info) < 0) return -1;
		if((fp = fopen(zipfname,"r")) == NULL) return -1;

	}else if(file_type < 0){
		return -1;
	}
	fstat(fileno(fp),&sbuf);

	xtext_usage(NULL,0);
	while(fgets(buf,BUFSIZ,fp) != NULL){
		total_size += strlen(buf);
		xtext_usage(XJP_MESSAGE[7],(double)total_size/sbuf.st_size);

		wkfConvertKanjiCodeOfString(KC_UNKNOWN,buf,KC_EUC,buf_euc,BUFSIZ);

		if(ruby_on && head_line < 30){
			if(strncmp(buf_euc,HEADER_LINE,strlen(HEADER_LINE)) == 0){
				HEAD = HEAD ? 0 : 1;
				head_line++;
				continue;
			}
			if(HEAD){head_line++; continue;}
		}
		head_line++;


		if((slen = _no_k(buf_euc)) == 0){
			_ibuf_add(head,tail,&ihead,&itail,0,total_line++); 
		}
		if(total_line == 1) strcpy(info->title,buf_euc);

		i=0;
		for(p=buf_euc;*p;p++){
			if(*p & 0x80){
				c1 = *p++;
				c2 = *p;

				if(ruby_on){

					if(RUB >= 0) RUB++;
			                if(c1 == 0xa1 && c2 == 0xc3){
						RUB = 0;
					        continue;
					}
					if(c1 == 0xa1 && c2 == 0xd4){
					        if(itail != NULL && RUB >= 0) itail->rub = RUB;
						RUB = -1;
					        RUBI = 1;
					        continue;
					}
					if(c1 == 0xa1 && c2 == 0xd5 && RUBI){
                  				RUBI = 0;
                              			continue;
					}

					if(c1 == 0xa1 && c2 == 0xce && CHU == 0){
					        if((*(p+1) == 0xa1 && *(p+2) == 0xf4) ||
				                   (*(p+1) == 0xa1 && *(p+2) == 0xf6) ){

                                                        CHU = 1;
							continue;
                                                }
                                        }
				        if(c1 == 0xa1 && c2 == 0xcf && CHU == 1){
						i = checkRubyString(i,&ihead,&itail,&SMALL);
					        CHU = 0;
					        continue;
					}
				}

				if(RUBI){
					itextDataInsert(&itail->rubi_head,&itail->rubi_tail,c1,c2,1,0,0);
				}else if(CHU){
					if(itail != NULL){
						itextDataInsert(&itail->chu_head,&itail->chu_tail,c1,c2,1,0,0);
					}else{
						i++;i++;

						itextDataInsert(&ihead,&itail,CHU_KEY_C1,CHU_KEY_C2,1,1,SMALL);
						itextDataInsert(&itail->chu_head,&itail->chu_tail,c1,c2,0,0,0);
					}

				}else{
					i++;i++;
					itextDataInsert(&ihead,&itail,c1,c2,1,0,SMALL);

				}

			}else{
				if(*p == '\t'){
					int xj;
					for(xj=0;xj<4;xj++){
						i++;
						c1 = SPACE;
						c2 = SPACE;

						itextDataInsert(&ihead,&itail,c1,c2,0,0,SMALL);
					}
				}else{
					i++;
					c1 = *p;
					c2 = SPACE;

					if(CHU)
						itextDataInsert(&itail->chu_head,&itail->chu_tail,c1,c2,0,0,SMALL);
					else
						itextDataInsert(&ihead,&itail,c1,c2,0,0,SMALL);
				}

			}
			
		}
		if(i > 0){
			_ibuf_add(head,tail,&ihead,&itail,i,total_line++); 
		}

	}

	fclose(fp);

	if(file_type == 1) unlink(zipfname);

	xtext_usage(NULL,0);

	return total_line;
}
